=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-03-27 22:19:57 +0000
+++ bzrlib/builtins.py	2006-03-28 23:30:02 +0000
@@ -417,6 +417,7 @@
     def run(self, location=None, remember=False, overwrite=False, revision=None, verbose=False):
         # FIXME: too much stuff is in the command class        
         tree_to = WorkingTree.open_containing(u'.')[0]
+        br_to = tree_to.branch
         stored_loc = tree_to.branch.get_parent()
         if location is None:
             if stored_loc is None:
@@ -425,8 +426,10 @@
                 print "Using saved location: %s" % stored_loc
                 location = stored_loc
 
+        if br_to.get_parent() is None or remember:
+            br_to.set_parent(location)
+
         br_from = Branch.open(location)
-        br_to = tree_to.branch
 
         if revision is None:
             rev_id = None
@@ -437,9 +440,6 @@
 
         old_rh = br_to.revision_history()
         count = tree_to.pull(br_from, overwrite, rev_id)
-
-        if br_to.get_parent() is None or remember:
-            br_to.set_parent(location)
         note('%d revision(s) pulled.' % (count,))
 
         if verbose:

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2006-01-17 11:44:03 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2006-03-28 23:30:02 +0000
@@ -24,6 +24,8 @@
 
 from bzrlib.branch import Branch
 from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.osutils import abspath
+
 
 class TestPull(ExternalBase):
 
@@ -222,4 +224,39 @@
 
         self.assertEqual(rev_history_b, rev_history_a)
 
+    def test_pull_remember(self):
+        """Pull changes from one branch to another and test parent location."""
+        os.mkdir('a')
+        os.chdir('a')
 
+        self.example_branch()
+        self.runbzr('branch . ../b')
+        self.runbzr('branch . ../c')
+        file('bottles', 'wt').write('99 bottles of beer on the wall')
+        self.runbzr('add bottles')
+        self.runbzr('commit -m 99_bottles')
+        os.chdir('../b')
+        b = Branch.open('')
+        parent = b.get_parent()
+        # reset parent
+        b.set_parent(None)
+        self.assertEqual(None, b.get_parent())
+        # test pull for failure without parent set
+        out = self.runbzr('pull', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: No pull location known or specified.\n'))
+        # test implicit --remember when no parent set, this pull conflicts
+        file('bottles', 'wt').write('98 bottles of beer on the wall')
+        self.runbzr('add bottles')
+        self.runbzr('commit -m 98_bottles')
+        out = self.runbzr('pull ../a', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: These branches have diverged.  Try merge.\n'))
+        self.assertEquals(abspath(b.get_parent()), abspath(parent))
+        # test implicit --remember after resolving previous failure
+        self.runbzr('uncommit --force')
+        self.runbzr('pull')
+        self.assertEquals(abspath(b.get_parent()), abspath(parent))
+        # test explicit --remember
+        self.runbzr('pull ../c --remember')
+        self.assertEquals(abspath(b.get_parent()), abspath('../c'))

